<?php
namespace Controller;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Service;
use Util;
use Util\Messages;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\DevOps\Util\Utilities as DevopsUtilities;

class CommentType extends \Generated\Controller\CommentType {

    /** @var \Service\commentType $commentTypeService */
    protected $commentTypeService;

	public function __construct(\Service\CommentType $commentTypeService){
        $this->commentTypeService = $commentTypeService;
	}

    public function findCommentType(Request\Filter $request,Response $response){
        $conditions = $request->getFilterCriteria()->getConditions();
        $filter = DevopsUtilities::createQueryFiltered($conditions);
        $dataSet = $this->commentTypeService->findCommentType($filter);
        $response->addDataset(new DataSet('commentType', $dataSet));
    }

    public function saveCommentType(Request\DataSet $request,Response $response){
        try{
            $row = $request->getDataSet()->getRows()[0];
            $id = empty($row['ID']) ? null : $row['ID'];
            $name = empty($row['NAME']) ? null : $row['NAME'];
            $control = empty($row['CONTROL']) ? null : $row['CONTROL'];
            $attachmentRequired = empty($row['ATTACHMENT_REQUIRED']) ? null : $row['ATTACHMENT_REQUIRED'];
            $commentDescription = empty($row['COMMENT_DESCRIPTION']) ? null : $row['COMMENT_DESCRIPTION'];
            $this->commentTypeService->saveCommentType($id, $name, $control, $attachmentRequired, $commentDescription);
        } catch(\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function removeCommentType(Request\Row $request, Response $response) {
        try{
            $row = $request->getRow();
            $id = $row['ID'];
            $this->commentTypeService->removeCommentType($id);
        } catch(\Exception $e) {
            $errorMessage = $e->getMessage();
            $isUsed = strpos($errorMessage, "ORA-02292");
            if($isUsed){
                $response->setError(new Response\Error(Messages::NOT_POSSIBLE_DELETE_COMMENT_TYPE, $e->getCode()));
            }else {
                $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
            }
        }
    }
}
