<?php
/**
 * Created by PhpStorm.
 * User: tuliomagalhaes
 * Date: 22/10/2015
 * Time: 12:22
 */

namespace Service;


use Doctrine\ORM\EntityManager;
use Util\PmCache;
use Util\Utilities;
use Zeedhi\DevOps\Model\Projectmanagement\Entities;

class CalendarDates
{

    /**
     * @var EntityManager
     */
    private $entityManager;
    /**
     * @var Utilities
     */
    private $environment;
    /**
     * @var \Factory\CalendarDates
     */
    private $calendarDatesFactory;
    /**
     * @var \Zeedhi\DevOps\Model\Projectmanagement\Repositories\CalendarDates
     */
    private $calendarDatesRepository;

    public function __construct(EntityManager $entityManager, Utilities $environment,
                                \Factory\CalendarDates $calendarDatesFactory)
    {

        $this->calendarDatesRepository = $entityManager
            ->getRepository(Utilities::REPOSITORY_DIR . 'CalendarDates');
        $this->entityManager = $entityManager;
        $this->environment = $environment;
        $this->calendarDatesFactory = $calendarDatesFactory;
    }

    public function setDatesByCalendarDetail(Entities\CalendarDetail $calendarDetail)
    {
         if($calendarDetail->getIsException() == CalendarDetail::EXCEPTION_TRUE){
            $recurrenceType = $calendarDetail->getRecurrenceUnit();

            $this->removeAllDatesByCalendarDetail($calendarDetail);
            if (CalendarDetail::RECURRENCE_TYPE_DAILY == $recurrenceType) {
                $calendarDates = $this->setRecurrenceDaily($calendarDetail);
            } else if (CalendarDetail::RECURRENCE_TYPE_WEEKLY == $recurrenceType) {
                $calendarDates = $this->setRecurrenceWeekly($calendarDetail);
            } else if (CalendarDetail::RECURRENCE_TYPE_MONTHLY == $recurrenceType) {
                $calendarDates = $this->setRecurrenceMonthly($calendarDetail);
            } else if (CalendarDetail::RECURRENCE_TYPE_YEARLY == $recurrenceType) {
                $calendarDates = $this->setRecurrenceYearly($calendarDetail);
            } else {
                throw new \Exception('Tipo de recorrencia não encontrada');
            }
        }else{
            $calendarDates = $this->setWorkWeek($calendarDetail);
        }
        return $calendarDates;
    }

    public function removeAllDatesByCalendarDetail($calendarDetail)
    {
        try {
            $calendarDates = $this->calendarDatesRepository->findBy(array('calendarDetail' => $calendarDetail));
            foreach ($calendarDates as $calendarDate) {
                $this->entityManager->remove($calendarDate);
            }
            PmCache::getInstance()->Clear();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function setRecurrenceDaily(Entities\CalendarDetail $calendarDetail)
    {
        $beginDate = $calendarDetail->getBeginDate()->format(Utilities::FORMAT_DATE_DEFAULT);
        $endDate = $calendarDetail->getEndDate()->format(Utilities::FORMAT_DATE_DEFAULT);
        $every = $calendarDetail->getEvery();
        $type = $calendarDetail->getType();
        $dailyHours = $calendarDetail->getDailyHours();
        $datesArray = $this->calendarDatesRepository->getDatesForCalendarDaily($beginDate, $endDate, $every,
            $calendarDetail->getBeginDate()->format('d'));
        $calendarDates = $this->createDates($calendarDetail, $datesArray, $type, $dailyHours);

        return $calendarDates;
    }

    public function setRecurrenceWeekly(Entities\CalendarDetail $calendarDetail)
    {
        $beginDate = $calendarDetail->getbeginDate()->format(Utilities::FORMAT_DATE_DEFAULT);
        $endDate = $calendarDetail->getendDate()->format(Utilities::FORMAT_DATE_DEFAULT);
        $every = $calendarDetail->getEvery();
        $type = $calendarDetail->getType();
        $dailyHours = $calendarDetail->getDailyHours();
        $calendarWeeklyDays = $calendarDetail->getCalendarWeeklyDays();
        $week = array();
        foreach ($calendarWeeklyDays as $calendarWeeklyDay) {
            $week[] = $calendarWeeklyDay->getWeekDay();
        }
        $weekdaysString = implode($week, ',');
        $datesArray = $this->calendarDatesRepository->getDatesForCalendarWeek($beginDate, $endDate, $every,
            $weekdaysString);
        $calendarDates = $this->createDates($calendarDetail, $datesArray, $type, $dailyHours);

        return $calendarDates;
    }

    public function setRecurrenceMonthly(Entities\CalendarDetail $calendarDetail)
    {
        $beginDate = $calendarDetail->getbeginDate()->format(Utilities::FORMAT_DATE_DEFAULT);
        $endDate = $calendarDetail->getendDate()->format(Utilities::FORMAT_DATE_DEFAULT);
        $every = $calendarDetail->getEvery();
        $type = $calendarDetail->getType();
        $dailyHours = $calendarDetail->getDailyHours();
        $orderDay = $calendarDetail->getOrderDay();
        $monthlyDay = $calendarDetail->getMonthDay();
        $month = $calendarDetail->getMonth();
        $weekDay = $calendarDetail->getWeekDay();
        if (empty($orderDay)) {
            $datesArray = $this->calendarDatesRepository->getDatesForMonthlyDayRecurrence($beginDate, $endDate,
                $monthlyDay, $every, $month);
        } else {
            $datesArray = $this->calendarDatesRepository->getDatesForMonthlyWeekRecurrence($beginDate, $endDate,
                $orderDay, $weekDay, $every, $month);
        }
        $calendarDates = $this->createDates($calendarDetail, $datesArray, $type, $dailyHours);

        return $calendarDates;
    }

    public function setRecurrenceYearly(Entities\CalendarDetail $calendarDetail)
    {
        $beginDate = $calendarDetail->getbeginDate()->format(Utilities::FORMAT_DATE_DEFAULT);
        $endDate = $calendarDetail->getendDate()->format(Utilities::FORMAT_DATE_DEFAULT);
        $type = $calendarDetail->getType();
        $dailyHours = $calendarDetail->getDailyHours();
        $orderDay = $calendarDetail->getOrderDay();
        $monthlyDay = $calendarDetail->getMonthDay();
        $month = $calendarDetail->getMonth();
        $weekDay = $calendarDetail->getWeekDay();
        if (empty($orderDay)) {
            $datesArray = $this->calendarDatesRepository->getDatesForYearlyDayRecurrence($beginDate, $endDate,
                $monthlyDay, $month);
        } else {
            $datesArray = $this->calendarDatesRepository->getDatesForYearlyWeekRecurrence($beginDate, $endDate,
                $orderDay, $weekDay, $month);
        }
        $calendarDates = $this->createDates($calendarDetail, $datesArray, $type, $dailyHours);

        return $calendarDates;
    }

    public function setWorkWeek(Entities\CalendarDetail $calendarDetail)
    {
        $calendarDates = array();
        if ($calendarDetail->getBeginDate()) {
            $beginDate = $calendarDetail->getbeginDate()->format(Utilities::FORMAT_DATE_DEFAULT);
            $endDate = $calendarDetail->getendDate()->format(Utilities::FORMAT_DATE_DEFAULT);
            $calendarWeeklyDays = $calendarDetail->getCalendarWeeklyDays();
            if($calendarWeeklyDays){
                foreach($calendarWeeklyDays as $calendarWeeklyDay){
                    $week = $calendarWeeklyDay->getWeekDay();
                    $hours = $calendarWeeklyDay->getDailyHours();
                    $type = $calendarWeeklyDay->getType();
                    $datesArray = $this->calendarDatesRepository->getDatesForWorkWeek($beginDate, $endDate, $week);
                    array_merge($calendarDates, $this->createDates($calendarDetail, $datesArray, $type, $hours));
                }
            }
        }
        return $calendarDates;
    }

    /**
     * @param Entities\CalendarDetail $calendarDetail
     * @param $datesArray
     * @param $type
     * @param $dailyHours
     * @return array
     */
    public function createDates(Entities\CalendarDetail $calendarDetail, $datesArray, $type, $dailyHours)
    {
        $calendarDates = array();
        foreach ($datesArray as $dateArray) {
            $date = \DateTime::createFromFormat(Utilities::FORMAT_DATE_DEFAULT_BY_BD, $dateArray['DATA']);
            $calendarDate = $this->calendarDatesFactory->createCalendarDate($calendarDetail, $date, $type, $status = 'A',
                $dailyHours);
            $this->entityManager->persist($calendarDate);
            array_push($calendarDates, $calendarDate);
        }
        PmCache::getInstance()->Clear();
        return $calendarDates;
    }

    public function findExceptionCalendarDates($calendarId,$beginDate,$endDate)
    {
        return $this->calendarDatesRepository->getExceptionCalendarDates($calendarId,$beginDate,$endDate);
    }
    public function findExceptionCalendarDatesByCalendarDefault($calendarId,$defaultCalendarId,$beginDate,$endDate)
    {
        return $this->calendarDatesRepository->getExceptionCalendarDatesByCalendarDefault($calendarId,$defaultCalendarId,$beginDate,$endDate);
    }
}