<?php
/**
 * Created by PhpStorm.
 * User: tuliomagalhaes
 * Date: 31/03/2015
 * Time: 16:01
 */

namespace Service;


use Util\InstanceProvider;
use Zeedhi\DevOps\Util\Utilities;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\Events\PreDispatch\Listener;
use Zeedhi\Framework\HTTP\Kernel;
use Zeedhi\Framework\Log\Exception;
use Zeedhi\Framework\Security\OAuth\OAuthImpl;
use Zeedhi\Framework\Session\Session;

class RequestListenerOAuth extends Listener {

    const OAUTH_HEADER = "OAuth-Token";
    const OAUTH_SECRET_ID = "OAuth-SecretId";
    const TOKEN_LOGIN =  "Token-login";
    const ORIGIN_HEADER = "Origin";
    const URL_DEVOPS_PRODUCTION = "http://devops.zeedhi.com";
    protected $appropriationValidationUrls;

    public function __construct(){
        $this->appropriationValidationUrls = $this->readAppropriationValidationRoutes();
        \Util\InstanceProvider::getLog()->setLog('instanciando RequestListenerOAuth',false,2);
    }


    private function readAppropriationValidationRoutes(){
        $urls = file_get_contents("../Json/appropriationValidationRoutes.json");
        return json_decode($urls, true);
    }

    private function readsendEmailIrregularities(){
        $urls = file_get_contents("../Json/sendEmailIrregularities.json");
        return json_decode($urls, true);
    }

    public function preDispatch(Request $request)
    {
        try {
            $this->loginOauth($request);
            $this->appropriationValidation($request);

        }catch (\Exception $e){
            throw new \Exception($e->getMessage());
        }
    }

    protected function loginOauth(Request $request)
    {
        $environment = InstanceProvider::getEnvironment();
        if(!$environment->getCurrentCdoperador()
            && false === strpos($request->getRoutePath(),'/lib_')
                && false === strpos($request->getRoutePath(),'/acl_zhu_')
        ){
            $userIdPm = $environment->getCurrentUserIdPm();
            $organizationIdPm = $environment->getCurrentOrganizationIdPm();
            InstanceProvider::getService()->getResource()->setSession($organizationIdPm, $userIdPm);
        }else if(!$environment->getCurrentUserId()){
            InstanceProvider::getService()->getResource()->setSessionByCdOperador($environment->getCurrentCdoperador());
        }
    }
    /**
     * @param Request $request
     * @throws \Exception
     */
    protected function appropriationValidation(Request $request)
    {
        if (in_array($request->getRoutePath(), $this->appropriationValidationUrls) &&
            InstanceProvider::getZeedhiFrameworkHTTPKernel()->getHttpRequest()->getHeaders()->get(self::ORIGIN_HEADER) != self::URL_DEVOPS_PRODUCTION
        ) {
            $appropriationValidation = InstanceProvider::getService()->getTask()->appropriationValidationOfLast30Day();
            if (!$appropriationValidation[0]['PREVIOUS_DAY_APPROPRIATION_OK']) {
                $name = $appropriationValidation[0]['USER_NAME'];
                $id = $appropriationValidation[0]['USER_ID'];
                $dateString = Utilities::getDateNowString();
                $routes = $request->getRoutePath();
                $message = "          <strong> Operação irregular!</strong>
                                          <br/> Usuário: $name com id $id
                                          <br/> na rota $routes na data $dateString";
                $emails = $this->readsendEmailIrregularities();
                InstanceProvider::getService()->getTask()->sendsMailIrregularities($emails, $message);
                throw new \Exception($message);
            }
        }
    }
}